{+--------------------------------------------------------------------------+
 | Component:   TmwComboBase
 | Created:     05.98 with help of Arabella by Thomas Quester
 |              Give it a try. http://members.tripod.com/~Arabella_fairy/
 | Author:      Martin Waldenburg
 | Copyright    1998, all rights reserved.
 | Description: This is intended to be a base for ComboBox like Components
 | Status:      FreeWare
 | Version:     1.0
 | DISCLAIMER:  This is provided as is, expressly without a warranty of any kind.
 |              You use it at your own risc.
 +--------------------------------------------------------------------------+}
unit mwCombBase;


interface

uses
  Windows, SysUtils, Messages, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, ExtCtrls, Buttons, Commctrl, Grids;

type
  TmwComboBase = class;

  TmwScreenBase = class(TStringGrid)
  private
    FMouseInControl: Boolean;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure WMMouseMove(var Message: TWMMouseMove); message WM_MOUSEMOVE;
    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
  end;

  TmwComboBase = class(TCustomEdit)
    Box: TSpeedButton;
    Grid: TStringGrid;
    procedure BoxClick(Sender: TObject);
  private
    FCanvas: TControlCanvas;
    fImage: TBitmap;
    fStateImage: TBitmap;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure WndProc(var Message: TMessage); override;
    procedure SetStateImage(NewValue: TBitmap);
    procedure SetImage(NewValue: TBitmap);
    function GetBoxWidth: Integer;
    procedure SetBoxWidth(NewValue: Integer);
    function GetBoxGlyph: TBitmap;
    procedure SetBoxGlyph(NewValue: TBitmap);
    function GetGridHeight:Integer;
    procedure SetGridHeight(NewValue:Integer);
  protected
    procedure CreateWindowHandle(const Params: TCreateParams); override;
    procedure SetEditRect;
    procedure SetBounds(Left, Top, Width, Height: Integer); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
  public
    Action: procedure of object;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DrawImage(Bitmap: TBitmap; Position: Integer);
  published
    property Font;
    property Text;
    property Image: TBitmap read fImage write SetImage;
    property StateImage: TBitmap read fStateImage write SetStateImage;
    property BoxWidth: Integer read GetBoxWidth write SetBoxWidth;
    property BoxGlyph: TBitmap read GetBoxGlyph write SetBoxGlyph;
    property GridHeight:Integer read GetGridHeight write SetGridHeight;
    property ShowHint;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('mw', [TmwComboBase]);
end; { Register }

constructor TmwScreenBase.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Enabled := False;
  Visible := False;
end; { Create }

procedure TmwScreenBase.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do
  begin Style := Style or WS_BORDER; ExStyle := WS_EX_TOOLWINDOW or
    WS_EX_TOPMOST; WindowClass.Style := CS_SAVEBITS;
  end;
end; { CreateParams }

procedure TmwScreenBase.CreateWnd;
begin
  inherited CreateWnd;
  Windows.SetParent(Handle, 0); CallWindowProc(DefWndProc, Handle, wm_SetFocus, 0, 0);
end; { CreateWnd }

procedure TmwScreenBase.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  FMouseInControl := True;
  ReleaseCapture;
end; { CMMouseEnter }

procedure TmwScreenBase.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  FMouseInControl := False;
  SetCaptureControl(Self);
end; { CMMouseLeave }

procedure TmwScreenBase.WMMouseMove(var Message: TWMMouseMove);
begin
  inherited;
end; { WMMouseMove }

procedure TmwScreenBase.WMLButtonDown(var Message: TWMLButtonDown);
begin
  inherited;
  Case FMouseInControl of
    False:
      begin
        ReleaseCapture;
        Enabled := False;
        Visible := False;
      end;
    True:
      begin
        SendMessage((Owner.Owner as TWinControl).Handle, WM_LBUTTONDOWN, 0, 0);
      end;
  end;
end; { WMLButtonDown }

destructor TmwComboBase.Destroy;
begin
  FCanvas.Free;
  inherited Destroy;
  FImage.free;
  FStateImage.free;
end; { Destroy }

constructor TmwComboBase.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 241;
  Height := 21;
  TabOrder := 0;

  FCanvas := TControlCanvas.Create;
  FCanvas.Control := Self;

  Box := TSpeedButton.Create(Self);
  Box.Parent := Self;

  FImage := TBitmap.Create;
  FImage.Transparent := True;
  FStateImage := TBitmap.Create;
  FStateImage.Transparent := True;

  Grid := TmwScreenBase.Create(Self);
  Grid.Parent := Self;
end; { Create }

procedure TmwComboBase.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or ES_MULTILINE or WS_CLIPCHILDREN;
end;

procedure TmwComboBase.CreateWnd;
begin
  inherited CreateWnd;
  SetEditRect;
end; { CreateWnd }

procedure TmwComboBase.CreateWindowHandle(const Params: TCreateParams);
begin
  inherited CreateWindowHandle(Params);

  with Box do
  begin
    Left := 220;
    Top := 0;
    Width := 17;
    Height := 17;
    Cursor := crArrow;
    OnClick := BoxClick;
  end; { Box }

  with Grid do
  begin
    Left := 0;
    Top := 0;
    Width := 0;
    Height := 120;
  end; { Grid }

end; { CreateWindowHandle }

procedure TmwComboBase.WMPaint(var Message: TWMPaint);
begin
  inherited PaintHandler(Message);
  if not FStateImage.Empty and not FImage.Empty then
  begin
    DrawImage(FStateImage, 3);
    DrawImage(FImage, FStateImage.Width + 6);
  end else
    if not FStateImage.Empty and FImage.Empty then
    begin
      DrawImage(FStateImage, 3);
    end else
      if FStateImage.Empty and not FImage.Empty then
      begin
        DrawImage(FImage, 3);
      end;
end; { WMPaint }

procedure TmwComboBase.WndProc(var Message: TMessage);
begin
  with Message do
    case Msg of
      WM_WINDOWPOSCHANGING: SetEditRect;
    end;
  inherited WndProc(Message);
end; { WndProc }

procedure TmwComboBase.SetEditRect;
var
  Loc: TRect;
begin
  if not FStateImage.Empty and not FImage.Empty then
  begin
    SetRect(Loc, FStateImage.Width + FImage.Width + 9, 0, ClientWidth - Box.Width - 2, ClientHeight + 1);
    SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@Loc));
  end else
    if not FStateImage.Empty and FImage.Empty then
    begin
      SetRect(Loc, FStateImage.Width + 6, 0, ClientWidth - Box.Width - 2, ClientHeight + 1);
      SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@Loc));
    end else
      if FStateImage.Empty and not FImage.Empty then
      begin
        SetRect(Loc, FImage.Width + 6, 0, ClientWidth - Box.Width - 2, ClientHeight + 1);
        SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@Loc));
      end else
        if FStateImage.Empty and FImage.Empty then
        begin
          SetRect(Loc, 0, 0, ClientWidth - Box.Width - 2, ClientHeight + 1);
          SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@Loc));
        end;
end;

procedure TmwComboBase.BoxClick(Sender: TObject);
var
  CP, SP: TPoint;
begin
  CP.X := Left;
  CP.Y := Top + Height;
  SP := (Owner as TForm).ClientToScreen(CP);

  with Grid do
  begin
    Left := SP.X;
    Top := SP.Y;
    Width := Self.Width;
    Enabled := True;
    Visible := True;
  end; { Grid }
  SetCaptureControl(Grid);
end; { BoxClick }

procedure TmwComboBase.SetBounds(Left, Top, Width, Height: Integer);
begin
  Case Parent <> nil of
    True:
      begin
        inherited SetBounds(Left, Top, Width, Height);
        with Box do
        begin
          Left := Self.Width - Box.Width - 4;
          Height := Self.Height - 4;
        end;
        SetEditRect;
      end;
    False: inherited SetBounds(Left, Top, Width, Height);
  end;
end; { SetBounds }

procedure TmwComboBase.DrawImage(Bitmap: TBitmap; Position: Integer);
var
  YPos: Integer;
begin
  YPos := ClientHeight div 2 - 7;
  FCanvas.Draw(Position, YPos, Bitmap);
end; { DrawImage }

procedure TmwComboBase.SetImage(NewValue: TBitmap);
begin
  FImage.Assign(NewValue);
  FImage.Transparent := True;
  SetEditRect;
  RePaint;
end; { SetImage }

procedure TmwComboBase.SetStateImage(NewValue: TBitmap);
begin
  FStateImage.Assign(NewValue);
  FStateImage.Transparent := True;
  SetEditRect;
  RePaint;
end; { SetStateImage }

function TmwComboBase.GetBoxWidth: Integer;
begin
  GetBoxWidth := Box.Width;
end; { GetBoxWidth }

procedure TmwComboBase.SetBoxWidth(NewValue: Integer);
begin
  Box.Width := NewValue;
  Box.Left := ClientWidth - Box.Width;
  SetEditRect;
end; { SetBoxWidth }

function TmwComboBase.GetBoxGlyph: TBitmap;
begin
  GetBoxGlyph := Box.Glyph;
end; { GetBoxGlyph }

procedure TmwComboBase.SetBoxGlyph(NewValue: TBitmap);
begin
  Box.Glyph := NewValue;
end; { SetBoxGlyph }

function TmwComboBase.GetGridHeight:Integer;
begin
  GetGridHeight:= Grid.Height;
end;   { GetGridHeight }

procedure TmwComboBase.SetGridHeight(NewValue:Integer);
begin
  Grid.Height:= NewValue;
end;   { SetGridHeight }

end.

